@@ -81,6 +81,10 @@ extern "C" {
 #  define MRN_HANDLER_HAVE_SET_HA_SHARE_REF
 #endif
 
+#if (MYSQL_VERSION_ID >= 100505) && defined(MRN_MARIADB_P)
+#  define MRN_HANDLER_DELETE_TABLE_HAVE_HTON_DROP_TABLE
+#endif
+
 #if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
 #  define MRN_BIG_TABLES
 #elif defined(BIG_TABLES)
