@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2011-2020 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2020 Horimoto Yasuhiro <horimoto@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -209,7 +210,10 @@
   typedef MYSQL_LEX_STRING mrn_thd_lex_string;
 #endif
 
-#if defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100306
+#if defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100504
+#  define mrn_init_alloc_root(root, name, block_size, pre_alloc_size, flags) \
+  init_alloc_root(mrn_memory_key, root, block_size, pre_alloc_size, flags)
+#elif defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100306
 #  define mrn_init_alloc_root(root, name, block_size, pre_alloc_size, flags) \
   init_alloc_root(root, name, block_size, pre_alloc_size, flags)
 #elif defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100000
@@ -310,7 +314,8 @@ typedef HASH mrn_table_def_cache_type;
 #  define MRN_SEVERITY_WARNING Sql_condition::WARN_LEVEL_WARN
 #endif
 
-#if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
+#if (MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)) || \
+  (MYSQL_VERSION_ID >= 100504 && defined(MRN_MARIADB_P))
 #  define MRN_HAVE_PSI_MEMORY_KEY
 #endif
 
@@ -389,6 +394,10 @@ typedef HASH mrn_table_def_cache_type;
 #  define MRN_HAVE_MY_BYTEORDER_H
 #endif
 
+#if (defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100504)
+#  define MRN_HAVE_SQL_TYPE_GEOM_H
+#endif
+
 #if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
 #  define MRN_HAVE_SPATIAL
 #elif defined(HAVE_SPATIAL)
@@ -425,8 +434,23 @@ typedef HASH mrn_table_def_cache_type;
   ((select_lex)->options)
 #endif
 
+#if MYSQL_VERSION_ID >= 80021 && !defined(MRN_MARIADB_P)
+#  define MRN_SELECT_LEX_GET_FIELDS_LIST(select_lex) \
+  ((select_lex)->fields_list)
+#else
+#  define MRN_SELECT_LEX_GET_FIELDS_LIST(select_lex) \
+  ((select_lex)->item_list)
+#endif
+
 #if defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100000
-#  if MYSQL_VERSION_ID >= 100306
+#  if MYSQL_VERSION_ID >= 100504
+#    define mrn_init_sql_alloc(thd, name, mem_root)                     \
+  init_sql_alloc(mrn_memory_key,                                        \
+                 mem_root,                                              \
+                 TABLE_ALLOC_BLOCK_SIZE,                                \
+                 0,                                                     \
+                 MYF(thd->slave_thread ? 0 : MY_THREAD_SPECIFIC))
+#  elif MYSQL_VERSION_ID >= 100306
 #    define mrn_init_sql_alloc(thd, name, mem_root)                     \
   init_sql_alloc(mem_root,                                              \
                  name,                                                  \
@@ -623,9 +647,10 @@ typedef HASH mrn_table_def_cache_type;
 #endif
 
 #ifdef MRN_PERCONA_P
-#  if ((MYSQL_VERSION_ID >= 50634) && (MYSQL_VERSION_ID < 50700)) || \
-      (MYSQL_VERSION_ID >= 50721)
-#    define MRN_HAVE_DB_TYPE_ROCKSDB
+#  if MYSQL_VERSION_ID < 80000 && \
+  (((MYSQL_VERSION_ID >= 50634) && (MYSQL_VERSION_ID < 50700)) ||       \
+   (MYSQL_VERSION_ID >= 50721))
+#    define MRN_NEED_ROCKSDB_DB_TYPE_FIX
 #  endif
 #endif
 
@@ -761,11 +786,13 @@ typedef HASH mrn_table_def_cache_type;
 #  endif
 #endif
 
-#define mrn_thd_set_ha_data(thd, hton, ha_data) \
+#if MYSQL_VERSION_ID >= 100504 && defined(MRN_MARIADB_P)
+#  define mrn_thd_set_ha_data(thd, hton, ha_data) \
+  thd_set_ha_data(thd, hton, ha_data)
+#else
+#  define mrn_thd_set_ha_data(thd, hton, ha_data) \
   *thd_ha_data(thd, hton) = ha_data
-// TODO
-/* #define mrn_thd_set_ha_data(thd, hton, ha_data) \ */
-/*    thd_set_ha_data(thd, hton, ha_data) */
+#endif
 
 #if MYSQL_VERSION_ID >= 80011 && !defined(MRN_MARIADB_P)
 #  define mrn_destroy(pointer) destroy(pointer)
@@ -856,6 +883,30 @@ typedef HASH mrn_table_def_cache_type;
   ((field)->get_date((time), 0))
 #endif
 
+#if MYSQL_VERSION_ID >= 80021 && !defined(MRN_MARIADB_P)
+#  define MRN_FIELD_IS_UNSIGNED(field) \
+  ((field)->is_unsigned())
+#  define MRN_FIELD_FIELD_INDEX(field) \
+  ((field)->field_index())
+#  define MRN_FIELD_ALL_FLAGS(field) \
+  ((field)->all_flags())
+#  define MRN_FIELD_FIELD_PTR(field) \
+  ((field)->field_ptr())
+#  define MRN_FIELD_SET_FIELD_PTR(field, new_ptr) \
+  ((field)->set_field_ptr((new_ptr)))
+#else
+#  define MRN_FIELD_IS_UNSIGNED(field) \
+  ((field)->unsigned_flag)
+#  define MRN_FIELD_FIELD_INDEX(field) \
+  ((field)->field_index)
+#  define MRN_FIELD_ALL_FLAGS(field) \
+  ((field)->flags)
+#  define MRN_FIELD_FIELD_PTR(field) \
+  ((field)->ptr)
+#  define MRN_FIELD_SET_FIELD_PTR(field, new_ptr) \
+  (field)->ptr = (new_ptr)
+#endif
+
 #if defined(MRN_MARIADB_P) && (MYSQL_VERSION_ID >= 100400)
 #  define MRN_TABLE_RESET(table) ((table)->reset())
 #  define MRN_TABLE_SHARE_RESET(table_share) ((table_share)->reset())
